@@ -1,8 +1,9 @@
-# Copyright 2001-2004 The Apache Software Foundation
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
+# Licensed to the Apache Software Foundation (ASF) under one or more
+# contributor license agreements.  See the NOTICE file distributed with
+# this work for additional information regarding copyright ownership.
+# The ASF licenses this file to You under the Apache License, Version 2.0
+# (the "License"); you may not use this file except in compliance with
+# the License.  You may obtain a copy of the License at
 #
 #     http://www.apache.org/licenses/LICENSE-2.0
 #
@@ -197,7 +198,10 @@ sub cmodules_write_makefile_aix {
     my $makefile = catfile $mod->{dir}, 'Makefile';
     my $apxsflags = '';
 
-    if ($dversion =~ 'APACHE1') {
+    #
+    # Only do this for Apache 1.*
+    #
+    if ($self->server->{rev} == 1) {
         $apxsflags = "-Wl,-bE:$name.exp";
         my $expfile = catfile $mod->{dir}, "$name.exp";
         if (! -f $expfile) {
@@ -238,9 +242,10 @@ sub cmodules_write_makefile_MSWin32 {
     my $name = $mod->{name};
     my $makefile = "$mod->{dir}/Makefile";
     debug "writing $makefile";
+    my $extras = '';
 
     my $lib = $self->cmodules_build_so($name);
-    my $extras = ' -llibhttpd -p ';
+    $extras = ' -llibhttpd -p ' if ($self->server->{rev} != 1);
     my $goners = join ' ', (map {$name . '.' . $_} qw(exp lib so lo));
 
     my $fh = Symbol::gensym();
@@ -342,6 +347,11 @@ my @cmodule_config_defines = map {
     cmodule_define($_);
 } @cmodule_config_names;
 
+my $cmodule_config_extra = 
+    "#ifndef APACHE_HTTPD_TEST_EXTRA_HOOKS\n".
+    "#define APACHE_HTTPD_TEST_EXTRA_HOOKS(p) do { } while (0)\n".
+    "#endif\n";
+
 my $cmodule_config_hooks = join ",\n    ", map {
     cmodule_define_name($_);
 } @cmodule_config_names;
@@ -392,6 +402,7 @@ my $cmodule_template_2 = <<"EOF";
 static void name ## _register_hooks(apr_pool_t *p)
 {
 @cmodule_hooks
+    APACHE_HTTPD_TEST_EXTRA_HOOKS(p);
 }
 
 module AP_MODULE_DECLARE_DATA name ## _module = {
@@ -431,6 +442,8 @@ sub cmodules_generate_include {
 
     print $fh @cmodule_hook_defines, @cmodule_config_defines;
 
+    print $fh $cmodule_config_extra;
+
     print $fh $self->cmodules_module_template;
 
     close $fh;
@@ -456,6 +469,8 @@ __DATA__
 #define AP_METHOD_BIT  1
 typedef size_t apr_size_t;
 typedef array_header apr_array_header_t;
+#define APR_OFF_T_FMT "ld"
+#define APR_SIZE_T_FMT "lu"
 #endif /* APACHE1 */
 
 #ifdef APACHE2
