@@ -1,8 +1,9 @@
-# Copyright 2001-2004 The Apache Software Foundation
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
+# Licensed to the Apache Software Foundation (ASF) under one or more
+# contributor license agreements.  See the NOTICE file distributed with
+# this work for additional information regarding copyright ownership.
+# The ASF licenses this file to You under the Apache License, Version 2.0
+# (the "License"); you may not use this file except in compliance with
+# the License.  You may obtain a copy of the License at
 #
 #     http://www.apache.org/licenses/LICENSE-2.0
 #
@@ -29,7 +30,7 @@ use Cwd;
 use ExtUtils::MakeMaker;
 use File::Find qw(finddepth);
 use File::Path;
-use File::Spec::Functions qw(catfile catdir);
+use File::Spec::Functions qw(catfile catdir canonpath);
 use File::Basename qw(basename dirname);
 use Getopt::Long qw(GetOptions);
 use Config;
@@ -38,18 +39,18 @@ use constant IS_APACHE_TEST_BUILD => Apache::TestConfig::IS_APACHE_TEST_BUILD;
 
 use constant STARTUP_TIMEOUT => 300; # secs (good for extreme debug cases)
 
-use constant CUSTOM_CONFIG_FILE => 'Apache/TestConfigData.pm';
-
 use subs qw(exit_shell exit_perl);
 
-my $original_command;
+my $orig_command;
 my $orig_cwd;
+my $orig_conf_opts;
 
 my %core_files  = ();
 my %original_t_perms = ();
 
 my @std_run      = qw(start-httpd run-tests stop-httpd);
-my @others       = qw(verbose configure clean help ssl http11 bugreport save);
+my @others       = qw(verbose configure clean help ssl http11 bugreport 
+                      save no-httpd one-process);
 my @flag_opts    = (@std_run, @others);
 my @string_opts  = qw(order trace);
 my @ostring_opts = qw(proxy ping);
@@ -57,47 +58,43 @@ my @debug_opts   = qw(debug);
 my @num_opts     = qw(times);
 my @list_opts    = qw(preamble postamble breakpoint);
 my @hash_opts    = qw(header);
-my @help_opts    = qw(clean help ping);
-my @exit_opts    = (@help_opts, @debug_opts);
+my @help_opts    = qw(clean help);
 my @request_opts = qw(get post head);
 
+my @exit_opts_no_need_httpd = (@help_opts);
+my @exit_opts_need_httpd    = (@debug_opts, qw(ping));
+
 my %usage = (
    'start-httpd'     => 'start the test server',
    'run-tests'       => 'run the tests',
    'times=N'         => 'repeat the tests N times',
-   'order=mode'      => 'run the tests in one of the modes: (repeat|rotate|random|SEED)',
+   'order=mode'      => 'run the tests in one of the modes: ' .
+                        '(repeat|rotate|random|SEED)',
    'stop-httpd'      => 'stop the test server',
+   'no-httpd'        => 'run the tests without configuring or starting httpd',
    'verbose[=1]'     => 'verbose output',
-   'configure'       => 'force regeneration of httpd.conf (tests will not be run)',
+   'configure'       => 'force regeneration of httpd.conf ' .
+                        ' (tests will not be run)',
    'clean'           => 'remove all generated test files',
    'help'            => 'display this message',
    'bugreport'       => 'print the hint how to report problems',
    'preamble'        => 'config to add at the beginning of httpd.conf',
    'postamble'       => 'config to add at the end of httpd.conf',
    'ping[=block]'    => 'test if server is running or port in use',
-   'debug[=name]'    => 'start server under debugger name (e.g. gdb, ddd, ...)',
+   'debug[=name]'    => 'start server under debugger name (gdb, ddd, etc.)',
    'breakpoint=bp'   => 'set breakpoints (multiply bp can be set)',
-   'header'          => "add headers to (".join('|', @request_opts).") request",
+   'header'          => "add headers to (" .
+                         join('|', @request_opts) . ") request",
    'http11'          => 'run all tests with HTTP/1.1 (keep alive) requests',
    'ssl'             => 'run tests through ssl',
    'proxy'           => 'proxy requests (default proxy is localhost)',
-   'trace=T'         => 'change tracing default to: warning, notice, info, debug, ...',
+   'trace=T'         => 'change tracing default to: warning, notice, ' .
+                        'info, debug, ...',
    'save'            => 'save test paramaters into Apache::TestConfigData',
+   'one-process'     => 'run the server in single process mode',
    (map { $_, "\U$_\E url" } @request_opts),
 );
 
-# variables stored in $Apache::TestConfigData::vars
-my @data_vars_must = qw(httpd apxs);
-my @data_vars_opt  = qw(user group port);
-# mapping from $Apache::TestConfigData::vars to $ENV settings
-my %vars_to_env = (
-    httpd => 'APACHE_TEST_HTTPD',
-    apxs  => 'APACHE_TEST_APXS',
-    user  => 'APACHE_TEST_USER',
-    group => 'APACHE_TEST_GROUP',
-    port  => 'APACHE_TEST_PORT',
-);
-
 sub fixup {
     #make sure we use an absolute path to perl
     #else Test::Harness uses the perl in our PATH
@@ -146,8 +143,9 @@ sub split_test_args {
     my @leftovers = ();
     for (@$argv) {
         my $arg = $_;
-        #need the t/ for stat-ing, but don't want to include it in test output
-        $arg =~ s@^(?:\./)?t/@@;
+        # need the t/ (or t\) for stat-ing, but don't want to include
+        # it in test output
+        $arg =~ s@^(?:\.[\\/])?t[\\/]@@;
         my $file = catfile $t_dir, $arg;
         if (-d $file and $_ ne '/') {
             my @files = <$file/*.t>;
@@ -180,7 +178,7 @@ sub split_test_args {
         push @leftovers, $_;
     }
 
-    $self->{tests} = \@tests;
+    $self->{tests} = [ map { canonpath($_) } @tests ];
     $self->{argv}  = \@leftovers;
 }
 
@@ -190,7 +188,8 @@ sub die_on_invalid_args {
     # at this stage $self->{argv} should be empty
     my @invalid_argv = @{ $self->{argv} };
     if (@invalid_argv) {
-        error "unknown opts or test names: @invalid_argv\n-help will list options\n";
+        error "unknown opts or test names: @invalid_argv\n" .
+            "-help will list options\n";
         exit_perl 0;
     }
 
@@ -234,6 +233,7 @@ sub getopts {
     my $req_wanted_args = Apache::TestRequest::wanted_args();
     my @argv = ();
     my %req_args = ();
+
     while (@ARGV) {
         my $val = shift @ARGV;
         if ($val =~ /^--?(.+)/) { # must have a leading - or --
@@ -252,6 +252,9 @@ sub getopts {
         push @argv, $val;
     }
 
+    # save the orig args (make a deep copy)
+    $orig_conf_opts = { %conf_opts };
+
     # fixup the filepath options on win32 (spaces, short names, etc.)
     if (Apache::TestConfig::WIN32) {
         for my $key (keys %conf_opts) {
@@ -323,11 +326,12 @@ sub default_run_opts {
 
     unless (grep { exists $opts->{$_} } @std_run, @request_opts) {
         if (@$tests && $self->{server}->ping) {
-            #if certain tests are specified and server is running, dont restart
+            # if certain tests are specified and server is running,
+            # dont restart
             $opts->{'run-tests'} = 1;
         }
         else {
-            #default is server-server run-tests stop-server
+            #default is start-server run-tests stop-server
             $opts->{$_} = 1 for @std_run;
         }
     }
@@ -347,6 +351,12 @@ sub install_sighandlers {
 
     $SIG{__DIE__} = sub {
         return unless $_[0] =~ /^Failed/i; #dont catch Test::ok failures
+
+        # _show_results() calls die() under a few conditions, such as
+        # when no tests are run or when tests fail.  make sure the message
+        # is propagated back to the user.
+        print $_[0] if (caller(1))[3]||'' eq 'Test::Harness::_show_results';
+
         $server->stop(1) if $opts->{'start-httpd'};
         $server->failed_msg("error running tests");
         exit_perl 0;
@@ -365,19 +375,18 @@ sub install_sighandlers {
     #try to make sure we scan for core no matter what happens
     #must eval "" to "install" this END block, otherwise it will
     #always run, a subclass might not want that
-
     eval 'END {
-             return unless is_parent(); # because of fork
-             $self ||=
-                 Apache::TestRun->new(test_config => Apache::TestConfig->thaw);
-             {
-                 local $?; # preserve the exit status
-                 eval {
-                    $self->scan_core;
-                 };
-             }
-             $self->try_bug_report();
-         }';
+        return unless is_parent(); # because of fork
+        $self ||=
+            Apache::TestRun->new(test_config => Apache::TestConfig->thaw);
+        {
+            local $?; # preserve the exit status
+            eval {
+               $self->scan_core;
+            };
+        }
+        $self->try_bug_report();
+    }';
     die "failed: $@" if $@;
 
 }
@@ -395,7 +404,8 @@ sub refresh {
     my $self = shift;
     $self->opt_clean(1);
     $self->{conf_opts}->{save} = delete $self->{conf_opts}->{thaw} || 1;
-    $self->{test_config} = $self->new_test_config();
+    $self->{test_config} = $self->new_test_config()->httpd_config;
+    $self->{test_config}->{server}->{run} = $self;
     $self->{server} = $self->{test_config}->server;
 }
 
@@ -458,9 +468,16 @@ sub configure_opts {
     $test_config->postamble_register($postamble);
 }
 
+sub pre_configure { }
+
 sub configure {
     my $self = shift;
 
+    if ($self->{opts}->{'no-httpd'}) {
+        warning "skipping httpd configuration";
+        return;
+    }
+
     # create the conf dir as early as possible
     $self->{test_config}->prepare_t_conf();
 
@@ -493,15 +510,17 @@ sub configure {
     # custom config save if
     # 1) requested to save
     # 2) no saved config yet
-    if ($self->{opts}->{save} or !custom_config_exists()) {
-        $self->custom_config_save();
+    if ($self->{opts}->{save} or
+        !Apache::TestConfig::custom_config_exists()) {
+        $test_config->custom_config_save($self->{conf_opts});
     }
 }
 
 sub try_exit_opts {
     my $self = shift;
+    my @opts = @_;
 
-    for (@exit_opts) {
+    for (@opts) {
         next unless exists $self->{opts}->{$_};
         my $method = "opt_$_";
         my $rc = $self->$method();
@@ -516,9 +535,8 @@ sub try_exit_opts {
         }
         else {
             warning "server $self->{server}->{name} is not running";
-            # cleanup a stale httpd.pid file if found
-            my $t_logs  = $self->{test_config}->{vars}->{t_logs};
-            my $pid_file = catfile $t_logs, "httpd.pid";
+            # cleanup a stale pid file if found
+            my $pid_file  = $self->{test_config}->{vars}->{t_pid_file};
             unlink $pid_file if -e $pid_file;
         }
         exit_perl $ok;
@@ -528,22 +546,6 @@ sub try_exit_opts {
 sub start {
     my $self = shift;
 
-    my $test_config = $self->{test_config};
-
-    unless ($test_config->{vars}->{httpd} or $test_config->{vars}->{apxs}) {
-        $self->opt_clean(1);
-        # this method restarts the whole program via exec
-        # so it never returns
-        $self->custom_config_first_time;
-    }
-
-    # if we have gotten that far we know at least about the location
-    # of httpd and or apxs, so let's save it if we haven't saved any
-    # custom configs yet
-    unless (custom_config_exists()) {
-        $self->custom_config_save();
-    }
-
     my $opts = $self->{opts};
     my $server = $self->{server};
 
@@ -551,7 +553,8 @@ sub start {
     my $file = $server->debugger_file;
     if (-e $file and $opts->{'start-httpd'}) {
         if ($server->ping) {
-            warning "server is running under the debugger, defaulting to -run";
+            warning "server is running under the debugger, " .
+                "defaulting to -run";
             $opts->{'start-httpd'} = $opts->{'stop-httpd'} = 0;
         }
         else {
@@ -614,6 +617,17 @@ sub new_test_config {
 sub set_ulimit_via_sh {
     return if Apache::TestConfig::WINFU;
     return if $ENV{APACHE_TEST_ULIMIT_SET};
+
+    # only root can allow unlimited core dumps on Solaris (8 && 9?)
+    if (Apache::TestConfig::SOLARIS) {
+        my $user = getpwuid($>) || '';
+        if ($user ne 'root') {
+            warning "Skipping 'set unlimited ulimit for coredumps', " .
+                "since we are running as a non-root user on Solaris";
+            return;
+        }
+    }
+
     my $binsh = '/bin/sh';
     return unless -e $binsh;
     $ENV{APACHE_TEST_ULIMIT_SET} = 1;
@@ -630,10 +644,10 @@ sub set_ulimit_via_sh {
     }
     close $sh;
 
-    $original_command = "ulimit -c unlimited; $original_command";
-    warning "setting ulimit to allow core files\n$original_command";
-    exec $original_command;
-    die "exec $original_command has failed"; # shouldn't be reached
+    $orig_command = "ulimit -c unlimited; $orig_command";
+    warning "setting ulimit to allow core files\n$orig_command";
+    # use 'or die' to avoid warnings due to possible overrides of die
+    exec $orig_command or die "exec $orig_command has failed";
 }
 
 sub set_ulimit {
@@ -663,32 +677,43 @@ sub run {
 
     # reconstruct argv, preserve multiwords args, eg 'PerlTrace all'
     my $argv = join " ", map { /^-/ ? $_ : qq['$_'] } @ARGV;
-    $original_command = "$^X $0 $argv";
+    $orig_command = "$^X $0 $argv";
     $orig_cwd = Cwd::cwd();
     $self->set_ulimit;
     $self->set_env; #make sure these are always set
 
+    $self->detect_relocation($orig_cwd);
+
     my(@argv) = @_;
 
     $self->getopts(\@argv);
 
     # must be called after getopts so the tracing will be set right
-    custom_config_load();
+    Apache::TestConfig::custom_config_load();
 
-    $self->pre_configure() if $self->can('pre_configure');
+    $self->pre_configure();
 
+    # can't setup the httpd-specific parts of the config object yet
     $self->{test_config} = $self->new_test_config();
 
-    # make it easy to move the whole distro
-    $self->refresh unless -e $self->{test_config}->{vars}->{top_dir};
-
     $self->warn_core();
 
+    # give TestServer access to our runtime configuration directives
+    # so we can tell the server stuff if we need to
+    $self->{test_config}->{server}->{run} = $self;
+
     $self->{server} = $self->{test_config}->server;
 
     local($SIG{__DIE__}, $SIG{INT});
     $self->install_sighandlers;
 
+    $self->try_exit_opts(@exit_opts_no_need_httpd);
+
+    # httpd is found here (unless it was already configured before)
+    $self->{test_config}->httpd_config();
+
+    $self->try_exit_opts(@exit_opts_need_httpd);
+
     if ($self->{opts}->{configure}) {
         warning "cleaning out current configuration";
         $self->opt_clean(1);
@@ -712,19 +737,101 @@ sub run {
         exit_perl 1;
     }
 
-    $self->try_exit_opts;
-
     $self->default_run_opts;
 
     $self->split_test_args;
 
     $self->die_on_invalid_args;
 
-    $self->start;
+    $self->start unless $self->{opts}->{'no-httpd'};
 
     $self->run_tests;
 
-    $self->stop;
+    $self->stop unless $self->{opts}->{'no-httpd'};
+}
+
+sub rerun {
+    my $vars = shift;
+
+    # in %$vars
+    # - httpd will be always set
+    # - apxs is optional
+
+    $orig_cwd ||= Cwd::cwd();
+    chdir $orig_cwd;
+    my $new_opts = " -httpd $vars->{httpd}";
+    $new_opts .= " -apxs $vars->{apxs}" if $vars->{apxs};
+
+    my $new_command = $orig_command;
+
+    # strip any old bogus -httpd/-apxs
+    $new_command =~ s/--?httpd\s+$orig_conf_opts->{httpd}//
+        if $orig_conf_opts->{httpd};
+    $new_command =~ s/--?httpd\s+$orig_conf_opts->{httpd}//
+        if $orig_conf_opts->{httpd} and $vars->{apxs};
+
+    # add new opts
+    $new_command .= $new_opts;
+
+    warning "running with new config opts: $new_command";
+
+    # use 'or die' to avoid warnings due to possible overrides of die
+    exec $new_command or die "exec $new_command has failed";
+}
+
+
+# make it easy to move the whole distro w/o running
+# 't/TEST -clean' before moving. when moving the whole package,
+# the old cached config will stay, so we want to nuke it only if
+# we realize that it's no longer valid. we can't just check the
+# existance of the saved top_dir value, since the project may have
+# been copied and the old dir could be still there, but that's not
+# the one that we work in
+sub detect_relocation {
+    my($self, $cur_top_dir) = @_;
+
+    my $config_file = catfile qw(t conf apache_test_config.pm);
+    return unless -e $config_file;
+
+    my %inc = %INC;
+    eval { require "$config_file" };
+    %INC = %inc; # be stealth
+    warn($@), return if $@;
+
+    my $cfg = 'apache_test_config'->new;
+
+    # if the top_dir from saved config doesn't match the current
+    # top_dir, that means that the whole project was relocated to a
+    # different directory, w/o running t/TEST -clean first (in each
+    # directory with a test suite)
+    my $cfg_top_dir = $cfg->{vars}->{top_dir};
+    return unless $cfg_top_dir;
+    return if $cfg_top_dir eq $cur_top_dir;
+
+    # if that's the case silently fixup the saved config to use the
+    # new paths, and force a complete cleanup. if we don't fixup the
+    # config files, the cleanup process won't be able to locate files
+    # to delete and re-configuration will fail
+    {
+        # in place editing
+        local @ARGV = $config_file;
+        local $^I = ".bak";  # Win32 needs a backup
+        while (<>) {
+            s{$cfg_top_dir}{$cur_top_dir}g;
+            print;
+        }
+        unlink $config_file . $^I;
+    }
+
+    my $cleanup_cmd = "$^X $0 -clean";
+    warning "cleaning up the old config";
+    # XXX: do we care to check success?
+    system $cleanup_cmd;
+
+    # XXX: I tried hard to accomplish that w/o starting a new process,
+    # but too many things get on the way, so for now just keep it as an
+    # external process, as it's absolutely transparent to the normal
+    # app-run
 }
 
 my @oh = qw(jeez golly gosh darn shucks dangit rats nuts dangnabit crap);
@@ -753,12 +860,13 @@ sub scan_core_incremental {
         require IO::Dir;
         my @cores = ();
         for (IO::Dir->new($vars->{t_dir})->read) {
-            next unless -f;
+            my $file = catfile $vars->{t_dir}, $_;
+            next unless -f $file;
             next unless /$core_pat/o;
-            my $core = catfile $vars->{t_dir}, $_;
-            next if exists $core_files{$core} && $core_files{$core} == -M $core;
-            $core_files{$core} = -M $core;
-            push @cores, $core;
+            next if exists $core_files{$file} &&
+                $core_files{$file} == -M $file;
+            $core_files{$file} = -M $file;
+            push @cores, $file;
         }
         return @cores 
             ? join "\n", "server dumped core, for stacktrace, run:",
@@ -881,6 +989,7 @@ sub adjust_t_perms {
 
         $self->check_perms($user, $uid, $gid);
 
+        $self->become_nonroot($user, $uid, $gid);
     }
 }
 
@@ -908,7 +1017,8 @@ sub run_root_fs_test {
     # setgroups() call as explained in perlvar.pod)
     my $groups = "$gid $gid";
     $( = $) = $groups;
-    die "failed to change gid to $gid" unless $( eq $groups && $) eq $groups;
+    die "failed to change gid to $gid"
+        unless $( eq $groups && $) eq $groups;
 
     # only now can change uid and euid
     $< = $> = $uid+0;
@@ -945,7 +1055,7 @@ sub check_perms {
     # test that the base dir is rwx by the selected non-root user
     my $vars = $self->{test_config}->{vars};
     my $dir  = $vars->{t_dir};
-    my $perl = $vars->{perl};
+    my $perl = Apache::TestConfig::shell_ready($vars->{perl});
 
     # find where Apache::TestRun was loaded from, so we load this
     # exact package from the external process
@@ -988,6 +1098,26 @@ EOI
     }
 }
 
+# in case the client side creates any files after the initial chown
+# adjustments we want the server side to be able to read/write them, so
+# they better be with the same permissions. dropping root permissions
+# and becoming the same user as the server side solves this problem.
+sub become_nonroot {
+    my ($self, $user, $uid, $gid) = @_;
+
+    warning "the client side drops 'root' permissions and becomes '$user'";
+
+    # first must change gid and egid ("$gid $gid" for an empty
+    # setgroups() call as explained in perlvar.pod)
+    my $groups = "$gid $gid";
+    $( = $) = $groups;
+    die "failed to change gid to $gid" unless $( eq $groups && $) eq $groups;
+
+    # only now can change uid and euid
+    $< = $> = $uid+0;
+    die "failed to change uid to $uid" unless $< == $uid && $> == $uid;
+}
+
 sub run_request {
     my($test_config, $opts) = @_;
 
@@ -1023,7 +1153,8 @@ sub opt_ping {
     my $server = $test_config->server;
     my $pid = $server->ping;
     my $name = $server->{name};
-    my $exit = not $self->{opts}->{'run-tests'}; #support t/TEST -ping=block -run ...
+    # support t/TEST -ping=block -run ...
+    my $exit = not $self->{opts}->{'run-tests'};
 
     if ($pid) {
         if ($pid == -1) {
@@ -1130,12 +1261,12 @@ sub generate_script {
         $opts{file} ||= catfile 't', 'TEST';
     }
 
-    my $body = "BEGIN { eval { require blib; } }\n";
+    my $body = "BEGIN { eval { require blib && blib->import; } }\n";
 
-    $body .= Apache::TestConfig->modperl_2_inc_fixup;
-
-    if (@Apache::TestMM::Argv) {
-        $body .= "\n\%Apache::TestConfig::Argv = qw(@Apache::TestMM::Argv);\n";
+    my %args = @Apache::TestMM::Argv;
+    while (my($k, $v) = each %args) {
+        $v =~ s/\|/\\|/g;
+        $body .= "\n\$Apache::TestConfig::Argv{'$k'} = q|$v|;\n";
     }
 
     my $header = Apache::TestConfig->perlscript_header;
@@ -1147,10 +1278,11 @@ sub generate_script {
         $body .= "\n\npackage $class;\n" .
                  "sub bug_report { print '$report' }\n\n";
     }
-    
+
     $body .= "$class->new->run(\@ARGV);";
 
-    Apache::Test::config()->write_perlscript($opts{file}, $body);
+    Apache::Test::basic_config()->write_perlscript($opts{file},
+                                                   $body);
 }
 
 # in idiomatic perl functions return 1 on success and 0 on
@@ -1176,6 +1308,13 @@ sub exit_shell {
 sub skip_test_suite {
     my $no_doubt = shift;
 
+    # we can't prompt when STDIN is not attached to tty, unless we
+    # were told that's it OK via env var (in which case some program
+    # will feed the interactive prompts
+    unless (-t STDIN || $ENV{APACHE_TEST_INTERACTIVE_PROMPT_OK}) {
+        $no_doubt = 1;
+    }
+
     print qq[
 
 Running the test suite is important to make sure that the module that
@@ -1196,450 +1335,6 @@ developers of this module.
     exit_perl 1;
 }
 
-# called from Apache::TestConfig::new()
-sub custom_config_add_conf_opts {
-    my $args = shift;
-
-    return unless $Apache::TestConfigData::vars and 
-        keys %$Apache::TestConfigData::vars;
-
-    # the logic is quite complicated with 'httpd' and 'apxs', since
-    # one is enough to run the test suite, and we need to avoid the
-    # situation where both are saved in custom config but only one
-    # (let's say httpd) is overriden by the command line /env var and
-    # a hell may break loose if we take that overriden httpd value and
-    # also use apxs from custom config which could point to a different
-    # server. So if there is an override of apxs or httpd, do not use
-    # the custom config for apxs or httpd.
-    my $vars_must_overriden = grep {
-        $ENV{ $vars_to_env{$_} } || $args->{$_}
-    } @data_vars_must;
-
-    # mod_perl 2.0 build always knows the right httpd location (and
-    # optionally apxs)
-    $vars_must_overriden++ if Apache::TestConfig::IS_MOD_PERL_2_BUILD();
-
-    unless ($vars_must_overriden) {
-        for (@data_vars_must) {
-            next unless $Apache::TestConfigData::vars->{$_};
-            $args->{$_} = $Apache::TestConfigData::vars->{$_};
-        }
-    }
-
-    for (@data_vars_opt) {
-        next unless $Apache::TestConfigData::vars->{$_};
-        # env vars override custom config
-        my $env_value = $ENV{ $vars_to_env{$_} };
-        next unless defined $env_value and length $env_value;
-        $args->{$_} ||= $Apache::TestConfigData::vars->{$_};
-    }
-}
-
-### Permanent custom configuration functions ###
-
-# determine which configuration file Apache/TestConfigData.pm to use
-# (as there could be several). The order searched is:
-# 1) $ENV{HOME}/.apache-test/
-# 2) in @INC
-my $custom_config_path;
-sub custom_config_path {
-
-    return $custom_config_path if $custom_config_path;
-
-    my @inc  = ();
-
-    # XXX $ENV{HOME} isn't propagated in mod_perl
-    push @inc, catdir $ENV{HOME}, '.apache-test' if $ENV{HOME};
-
-    push @inc, @INC;
-
-    for (@inc) {
-        my $candidate = File::Spec->rel2abs(catfile $_, CUSTOM_CONFIG_FILE);
-        next unless -e $candidate;
-        return $custom_config_path = $candidate;
-    }
-
-    return '';
-}
-
-sub custom_config_exists {
-    # try to load custom config if it wasn't loaded yet (there are
-    # many entry points to this API)
-    custom_config_load();
-
-    # it's enough to check whether we have a custom
-    # config for 'httpd' or 'apxs'.
-    my $httpd = $Apache::TestConfigData::vars->{httpd} || '';
-    return 1 if $httpd && -e $httpd && -x _;
-
-    my $apxs = $Apache::TestConfigData::vars->{apxs} || '';
-    return 1 if $apxs && -e $apxs && -x _;
-
-    return 0;
-}
-
-# to be used only from Apache-Test/Makefile.PL to write the custom
-# configuration module so it'll be copied to blib during 'make' and
-# updated to use custom config data during 'make test' and then
-# installed system-wide via 'make install'
-#
-# it gets written only if the custom configuration didn't exist
-# already
-sub custom_config_file_stub_write {
-
-    return if custom_config_exists();
-
-    # It doesn't matter whether it gets written under modperl-2.0/lib
-    # or Apache-Test/lib root, since Apache::TestRun uses the same
-    # logic and will update that file with real config data, which
-    # 'make install' will then pick and install system-wide. but
-    # remember that $FindBin::Bin is the location of top-level
-    # 'Makefile.PL'
-    require FindBin; # load it only for this particular use
-    my $path = catfile $FindBin::Bin, "lib",
-        Apache::TestRun::CUSTOM_CONFIG_FILE;
-
-    # write an empty stub
-    Apache::TestRun::custom_config_write($path, '');
-}
-
-
-sub custom_config_save {
-    my $self = shift;
-
-    my $vars = $self->{test_config}->{vars};
-    my $conf_opts = $self->{conf_opts};
-    my $config_dump = '';
-
-    # minimum httpd and/or apxs needs to be set
-    return 0 unless $vars->{httpd} or $Apache::TestConfigData::vars->{httpd}
-        or          $vars->{apxs}  or $Apache::TestConfigData::vars->{apxs};
-
-    # it doesn't matter how these vars were set (httpd may or may not get set
-    # using the path to apxs, w/o an explicit -httpd value)
-    for (@data_vars_must) {
-        next unless my $var = $vars->{$_} || $conf_opts->{$_};
-        $config_dump .= qq{    '$_' => '$var',\n};
-    }
-
-    # save these vars only if they were explicitly set via command line
-    # options. For example if someone builds A-T as user 'foo', then
-    # installs it as root and we save it, all users will now try to 
-    # configure under that user 'foo' which won't quite work.
-    for (@data_vars_opt) {
-        next unless my $var = $conf_opts->{$_};
-        $config_dump .= qq{    '$_' => '$var',\n};
-    }
-
-    if (IS_APACHE_TEST_BUILD) {
-        my $path = catfile $vars->{top_dir}, 'lib', CUSTOM_CONFIG_FILE;
-        # if it doesn't exist, then we already have a global config file
-        # if it does, then we have need to update it and its blib/ copy
-        if (-e $path and custom_config_path_is_writable($path)) {
-            custom_config_write($path, $config_dump);
-            # also update blib/lib, since usually that's the one that
-            # appears in @INC when t/TEST is run. and it won't be
-            # synced with blib/ unless 'make' was run
-            my $blib_path = catfile $vars->{top_dir},
-                'blib', 'lib', CUSTOM_CONFIG_FILE;
-            if (-e $blib_path and custom_config_path_is_writable($blib_path)) {
-                custom_config_write($blib_path, $config_dump);
-            }
-            return 1;
-        }
-    }
-
-    my $path;
-    if ($path = custom_config_path() ) {
-        # do nothing, the config file already exists (global)
-        debug "Found custom config '$path'";
-    }
-    elsif (File::Spec->file_name_is_absolute(__FILE__)) {
-        # next try a global location, as if it was configured before
-        # Apache::Test's 'make install' (install in the same dir as
-        # Apache/TestRun.pm)
-        # if the filename is not absolute that means that we are still
-        # in Apache-Test build (could just test for IS_APACHE_TEST_BUILD)
-        my $base = dirname dirname __FILE__;
-        $path = catdir $base, CUSTOM_CONFIG_FILE;
-    }
-
-    # check whether we can write to the directory of the chosen path
-    # (e.g. root-owned directory)
-    if ($path and custom_config_path_is_writable($path)) {
-        custom_config_write($path, $config_dump);
-        return 1;
-    }
-    # if we have no writable path yet, try to use ~
-    elsif ($ENV{HOME}) {
-        $path = catfile $ENV{HOME}, '.apache-test', CUSTOM_CONFIG_FILE;
-        if ($path and custom_config_path_is_writable($path)) {
-            custom_config_write($path, $config_dump);
-            return 1;
-        }
-    }
-
-    # XXX: should we croak since we failed to write config
-    error "Failed to find a config file to save the custom " .
-        "configuration in";
-    return 0;
-}
-
-sub custom_config_path_is_writable {
-    my $path = shift;
-
-    return 0 unless $path;
-
-    my $file_created    = '';
-    my $top_dir_created = '';
-    # first make sure that the file is writable if it exists
-    # already (it might be non-writable if installed via EU::MM or in
-    # blib/)
-    if (-e $path) {
-        my $mode = (stat _)[2];
-        $mode |= 0200;
-        chmod $mode, $path; # it's ok if we fail
-        # keep it writable if we have changed it from not being one
-        # so that custom_config_save will be able to just overwrite it
-    }
-    else {
-        my $dir = dirname $path;
-        if ($dir and !-e $dir) {
-            my @dirs = File::Path::mkpath($dir, 0, 0755);
-            # the top level dir to nuke on cleanup if it was created
-            $top_dir_created = shift @dirs if @dirs;
-        }
-        # not really create yet, but will be in the moment
-        $file_created = 1;
-    }
-
-    # try to open for append (even though it may not exist
-    my $fh = Symbol::gensym;
-    if (open $fh, ">>$path") {
-        close $fh;
-        # cleanup if we just created the file
-        unlink $path if $file_created;
-        File::Path::rmtree([$top_dir_created], 0, 0) if $top_dir_created;
-        return 1;
-    }
-
-    return 0;
-}
-
-sub custom_config_write {
-    my($path, $config_dump) = @_;
-
-    my $pkg = << "EOC";
-package Apache::TestConfigData;
-
-use strict;
-use warnings;
-
-\$Apache::TestConfigData::vars = {
-$config_dump
-};
-
-1;
-
-=head1 NAME
-
-Apache::TestConfigData - Configuration file for Apache::Test
-
-=cut
-EOC
-
-    debug "Writing custom config $path";
-    my $dir = dirname $path;
-    File::Path::mkpath($dir, 0, 0755) unless -e $dir;
-    my $fh = Symbol::gensym;
-    open $fh, ">$path" or die "Cannot open $path: $!";
-    print $fh $pkg;
-    close $fh;
-}
-
-my $custom_config_loaded = 0;
-sub custom_config_load {
-    debug "trying to load custom config data";
-
-    return if $custom_config_loaded;
-
-    if (my $custom_config_path = custom_config_path()) {
-        debug "loading custom config path '$custom_config_path'";
-        $custom_config_loaded++;
-        require $custom_config_path;
-    }
-}
-
-sub custom_config_first_time {
-    my $self = shift;
-
-    my $test_config = $self->{test_config};
-    my $vars = $test_config->{vars};
-
-    require File::Spec;
-    local *which = \&Apache::TestConfig::which;
-
-    print qq[
-
-We are now going to configure the Apache-Test framework.
-This configuration process needs to be done only once.
-
-];
-
-    print qq[
-
-First we need to know where the 'httpd' executable is located.
-If you have more than one Apache server is installed, make sure
-you supply the path to the one you are going to use for testing.
-You can always override this setting at run time via the '-httpd'
-option. For example:
-
-  % t/TEST -httpd /path/to/alternative/httpd
-
-or via the environment variable APACHE_TEST_HTTPD. For example:
-
-  % APACHE_TEST_HTTPD=/path/to/alternative/httpd t/TEST
-
-If for some reason you want to skip the test suite, type: skip
-];
-
-    {
-        my %choices = ();
-        my @tries = qw(httpd httpd2);
-        # Win32 uses Apache or perhaps Apache2, not apache/apache2
-        push @tries, Apache::TestConfig::WIN32 ?
-            qw(Apache Apache2) : qw(apache apache2);
-        for (grep defined $_,
-             map({ catfile $vars->{$_}, $vars->{target} } qw(sbindir bindir)),
-             $test_config->default_httpd, which($vars->{target}),
-             $ENV{APACHE}, $ENV{APACHE2},
-             $ENV{APACHE_TEST_HTTPD}, $ENV{APACHE_TEST_HTTPD2},
-             map {which($_)} @tries) {
-            $choices{$_}++ if -e $_ && -x _;
-        }
-        my $optional = 0;
-        my $wanted = 'httpd';
-        $vars->{$wanted} = 
-            _custom_config_prompt_path($wanted, \%choices, $optional);
-    }
-
-    print qq[
-
-Next we need to know where the 'apxs' script is located. This script
-provides a lot of information about the apache installation, and makes
-it easier to find things. However it's not available on all platforms,
-therefore it's optional.
-
-If you don't have it installed it's not a problem. Just press Enter.
-
-Notice that if you have Apache 2.x installed that script could be
-called as 'apxs2'.
-
-If you have more than one Apache server is installed, make sure you
-supply the path to the apxs script you are going to use for testing.
-You can always override this setting at run time via the '-apxs'
-option. For example:
-
-  % t/TEST -apxs /path/to/alternative/apxs
-
-or via the environment variable APACHE_TEST_APXS. For example:
-
-  % APACHE_TEST_APXS=/path/to/alternative/apxs t/TEST
-
-];
-    {
-        my %choices = ();
-        for (grep defined $_,
-             map({ catfile $vars->{$_}, 'apxs' } qw(sbindir bindir)),
-             $test_config->default_apxs,
-             $ENV{APXS},  $ENV{APACHE_TEST_APXS},  which('apxs'),
-             $ENV{APXS2}, $ENV{APACHE_TEST_APXS2}, which('apxs2')) {
-            $choices{$_}++ if -e $_ && -x _;
-        }
-        my $optional = 1;
-        my $wanted = 'apxs';
-        $vars->{$wanted} = 
-            _custom_config_prompt_path($wanted, \%choices, $optional);
-    }
-
-    $self->custom_config_save();
-
-    # we probably could reconfigure on the fly ($self->configure), but
-    # the problem is various cached data which won't be refreshed. so
-    # the simplest is just to restart the run from scratch
-    chdir $orig_cwd;
-    warning "rerunning '$original_command' with new config opts";
-    exec $original_command;
-    die "exec $original_command has failed"; # shouldn't be reached
-}
-
-sub _custom_config_prompt_path {
-    my($wanted, $rh_choices, $optional) = @_;
-
-    my $ans;
-    my $default = '';
-    my $optional_str = $optional ? " (optional)" : '';
-    my $prompt =
-        "\nPlease provide a full path to$optional_str '$wanted' executable";
-
-    my @choices = ();
-    if (%$rh_choices) {
-        $prompt .= " or choose from the following options:\n\n";
-        my $c = 0;
-        for (sort keys %$rh_choices) {
-            $c++;
-            $prompt .= "    [$c] $_\n";
-            push @choices, $_;
-        }
-        $prompt .= " \n";
-        $default = 1; # a wild guess
-    }
-    else {
-        $prompt .= ":\n\n";
-    }
-
-    while (1) {
-        $ans = ExtUtils::MakeMaker::prompt($prompt, $default);
-
-        # strip leading/closing spaces
-        $ans =~ s/^\s*|\s*$//g;
-
-        # convert the item number to the path
-        if ($ans =~ /^(\d+)$/) {
-            if ($1 > 0 and $choices[$1-1]) {
-                $ans = $choices[$1-1];
-            }
-            else {
-                warn "The choice '$ans' doesn't exist\n";
-                next;
-            }
-        }
-
-        if ($optional) {
-            return '' unless $ans;
-        }
-
-        # stop the test suite without an error (so automatic tools
-        # like CPAN.pm will be able to continue)
-        if (lc($ans) eq 'skip' && !$optional) {
-            skip_test_suite();
-            next; # in case they change their mind
-        }
-
-        unless (File::Spec->file_name_is_absolute($ans)) {
-            my $cwd = Cwd::cwd();
-            warn "The path '$ans' is not an absolute path. " .
-                "Please specify an absolute path\n";
-            next;
-        }
-
-        warn("'$ans' doesn't exist\n"),     next unless -e $ans;
-        warn("'$ans' is not executable\n"), next unless -x $ans;
-
-        return $ans;
-    }
-}
-
 1;
 
 __END__
@@ -1712,10 +1407,16 @@ I<t/TEST.PL>:
       my $self = shift;
       # Don't load an installed mod_apreq
       Apache::TestConfig::autoconfig_skip_module_add('mod_apreq.c');
+  
+      $self->SUPER::pre_configure();
   }
 
 Notice that the extension is I<.c>, and not I<.so>.
 
+Don't forget to run the super class' c<pre_configure()> method.
+
+
+
 =head2 C<new_test_config>
 
 META: to be completed
@@ -1735,6 +1436,10 @@ C<APACHE_TEST_GROUP>) or by giving the relevant option (C<-httpd>,
 C<-apxs>, C<-port>, C<-user>, and C<-group>) when the C<TEST> script
 is run.
 
+To avoid either using previous persistent configurations or saving
+current configurations, set the C<APACHE_TEST_NO_STICKY_PREFERENCES>
+environment variable to a true value.
+
 Finally it's possible to permanently override the previously saved
 options by passing C<L<-save|/Saving_Custom_Configuration_Options>>.
 
