@@ -1,8 +1,9 @@
-# Copyright 2003-2005 The Apache Software Foundation
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
+# Licensed to the Apache Software Foundation (ASF) under one or more
+# contributor license agreements.  See the NOTICE file distributed with
+# this work for additional information regarding copyright ownership.
+# The ASF licenses this file to You under the Apache License, Version 2.0
+# (the "License"); you may not use this file except in compliance with
+# the License.  You may obtain a copy of the License at
 #
 #     http://www.apache.org/licenses/LICENSE-2.0
 #
@@ -26,6 +27,7 @@ use File::Find;
 use Apache2::Build ();
 use ModPerl::MM;
 use constant WIN32 => Apache2::Build::WIN32;
+use constant CYGWIN => Apache2::Build::CYGWIN;
 
 our %PM; #add files to installation
 
@@ -74,7 +76,9 @@ sub WriteMakefile {
     $build ||= build_config();
     ModPerl::MM::my_import(__PACKAGE__);
 
-    my $inc = $build->inc;
+    my $inc;
+    $inc = $args{INC} if $args{INC};
+    $inc = " " . $build->inc;
     if (my $glue_inc = $build->{MP_XS_GLUE_DIR}) {
         for (split /\s+/, $glue_inc) {
             $inc .= " -I$_";
@@ -83,6 +87,7 @@ sub WriteMakefile {
 
     my $libs;
     my @libs = ();
+    push @libs, $args{LIBS} if $args{LIBS};
     if (Apache2::Build::BUILD_APREXT) {
         # in order to decouple APR/APR::* from mod_perl.so,
         # link these modules against the static MP_APR_LIB lib,
@@ -91,29 +96,46 @@ sub WriteMakefile {
         # usual. This is done for APR in xs/APR/APR/Makefile.PL.
         my $name = $args{NAME};
         if ($name =~ /^APR::\w+$/) {
-            @libs = ($build->apache_libs, $build->mp_apr_lib);
+            # For cygwin compatibility, the order of the libs should be
+            # <mod_perl libs> <apache libs>
+            @libs = ($build->mp_apr_lib, $build->apache_libs);
         }
         else {
-            @libs = ($build->apache_libs, $build->modperl_libs);
+            @libs = ($build->modperl_libs, $build->apache_libs);
         }
     }
     else {
-        @libs = ($build->apache_libs, $build->modperl_libs);
+        @libs = ($build->modperl_libs, $build->apache_libs);
     }
     $libs = join ' ', @libs;
 
-    my $ccflags = $build->perl_ccopts . $build->ap_ccopts;
+    my $ccflags;
+    $ccflags = $args{CCFLAGS} if $args{CCFLAGS};
+    $ccflags = " " . $build->perl_ccopts . $build->ap_ccopts;
+
+    my $optimize;
+    $optimize = $args{OPTIMIZE} if $args{OPTIMIZE};
+    $optimize = " " . $build->perl_config('optimize');
+
+    my $lddlflags;
+    $lddlflags = $args{LDDLFLAGS} if $args{LDDLFLAGS};
+    $lddlflags = " " . $build->perl_config('lddlflags');
+
+    my %dynamic_lib;
+    %dynamic_lib = %{ $args{dynamic_lib}||{} } if $args{dynamic_lib};
+    $dynamic_lib{OTHERLDFLAGS} = $build->otherldflags;
 
     my @opts = (
-        INC       => $inc,
-        CCFLAGS   => $ccflags,
-        OPTIMIZE  => $build->perl_config('optimize'),
-        LDDLFLAGS => $build->perl_config('lddlflags'),
-        LIBS      => $libs,
-        dynamic_lib => { OTHERLDFLAGS => $build->otherldflags },
+        INC         => $inc,
+        CCFLAGS     => $ccflags,
+        OPTIMIZE    => $optimize,
+        LDDLFLAGS   => $lddlflags,
+        LIBS        => $libs,
+        dynamic_lib => \%dynamic_lib,
     );
 
     my @typemaps;
+    push @typemaps, $args{TYPEMAPS} if $args{TYPEMAPS};
     my $pwd = Cwd::fastcwd();
     for ('xs', $pwd, "$pwd/..") {
         my $typemap = $build->file_path("$_/typemap");
@@ -150,7 +172,7 @@ sub ModPerl::BuildMM::MY::constants {
         #skip .xs -> .so if we are linking static
         my $name = $self->{NAME};
         unless ($always_dynamic{$name}) {
-            if (my($xs) = keys %{ $self->{XS} }) {
+            if (my ($xs) = keys %{ $self->{XS} }) {
                 $self->{HAS_LINK_CODE} = 0;
                 print "$name will be linked static\n";
                 #propagate static xs module to src/modules/perl/Makefile
@@ -190,6 +212,7 @@ sub ModPerl::BuildMM::MY::postamble {
     # allow 'make -j'
     require ExtUtils::MakeMaker;
     my $mm_ver = $ExtUtils::MakeMaker::VERSION;
+    $mm_ver =~ s/_.*//; # handle dev versions like 6.30_01
     my $pm_to_blib = ($mm_ver >= 6.22 && $mm_ver <= 6.25)
         ? "pm_to_blib.ts"
         : "pm_to_blib";
@@ -233,7 +256,7 @@ sub ModPerl::BuildMM::MY::postamble {
             }
         }
 
-        while (my($pm, $blib) = each %pms) {
+        while (my ($pm, $blib) = each %pms) {
             $pm   =~ s|/\./|/|g; # clean the path
             $blib =~ s|/\./|/|g; # clean the path
             my @segm = splitdir $blib;
@@ -250,7 +273,8 @@ sub ModPerl::BuildMM::MY::postamble {
                     "-e ModPerl::BuildMM::glue_pod $pm $podpath $blib";
 
                 # Win32 doesn't normally install man pages
-                next if WIN32;
+                # and Cygwin doesn't allow '::' in file names
+                next if WIN32 || CYGWIN;
 
                 # manify while we're at it
                 my (undef, $man, undef) = $blib =~ m!(blib/lib/)(.*)(\.pm)!;
@@ -282,7 +306,7 @@ sub ModPerl::BuildMM::MY::postamble {
 sub glue_pod {
 
     die "expecting 3 arguments: pm, pod, dst" unless @ARGV == 3;
-    my($pm, $pod, $dst) = @ARGV;
+    my ($pm, $pod, $dst) = @ARGV;
 
     # it's possible that the .pm file is not existing
     # (e.g. ThreadMutex.pm is not created on unless
@@ -313,7 +337,7 @@ sub ModPerl::BuildMM::MY::post_initialize {
     $build ||= build_config();
     my $pm = $self->{PM};
 
-    while (my($k, $v) = each %PM) {
+    while (my ($k, $v) = each %PM) {
         if (-e $k) {
             $pm->{$k} = $v;
         }
@@ -331,7 +355,7 @@ sub ModPerl::BuildMM::MY::post_initialize {
 my $apr_config;
 
 sub ModPerl::BuildMM::MY::libscan {
-    my($self, $path) = @_;
+    my ($self, $path) = @_;
 
     $apr_config ||= $build->get_apr_config();
 
