@@ -1,8 +1,9 @@
-# Copyright 2001-2005 The Apache Software Foundation
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
+# Licensed to the Apache Software Foundation (ASF) under one or more
+# contributor license agreements.  See the NOTICE file distributed with
+# this work for additional information regarding copyright ownership.
+# The ASF licenses this file to You under the Apache License, Version 2.0
+# (the "License"); you may not use this file except in compliance with
+# the License.  You may obtain a copy of the License at
 #
 #     http://www.apache.org/licenses/LICENSE-2.0
 #
@@ -48,11 +49,15 @@ sub as_string {
 
     # httpd opts
     my $test_config = Apache::TestConfig->new({thaw=>1});
+
     if (my $httpd = $test_config->{vars}->{httpd}) {
         $command = "$httpd -V";
         $cfg .= "\n\n*** $command\n";
         $cfg .= qx{$command};
-    } else {
+
+        $cfg .= Apache::TestConfig::ldd_as_string($httpd);
+    } 
+    else {
         $cfg .= "\n\n*** The httpd binary was not found\n";
     }
 
