@@ -1,8 +1,9 @@
-/* Copyright 2000-2005 The Apache Software Foundation
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+/* Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -271,6 +272,13 @@ PerlInterpreter *modperl_startup(server_rec *s, apr_pool_t *p)
     PL_reentrant_buffer->_crypt_struct.current_saltbits = 0;
 #endif
 
+    /* We need to reset $0 to argv[0] (httpd) since perl_parse() will
+     * have set it to '-e'. Being magic-aware ensures that some
+     * OS-specific magic will happen (i.e. setproctitle() on *BSDs)
+     */
+    PL_origalen = strlen(argv[0]) + 1;
+    sv_setpv_mg(get_sv("0",0), argv[0]);
+
     perl_run(perl);
 
 #ifdef USE_ITHREADS
@@ -478,8 +486,8 @@ static int modperl_post_config_require(server_rec *s, apr_pool_t *p)
 #ifdef USE_ITHREADS
 static void modperl_init_clones(server_rec *s, apr_pool_t *p)
 {
-    modperl_config_srv_t *base_scfg = modperl_config_srv_get(s);
 #ifdef MP_TRACE
+    modperl_config_srv_t *base_scfg = modperl_config_srv_get(s);
     char *base_name = modperl_server_desc(s, p);
 #endif /* MP_TRACE */
 
@@ -695,9 +703,17 @@ static int modperl_hook_post_config_last(apr_pool_t *pconf, apr_pool_t *plog,
     }
 #endif
 
+#if PERL_REVISION == 5 && PERL_VERSION < 9
+#define MP_PERL_VERSION_STAMP "Perl/v%vd"
+#else
+#define MP_PERL_VERSION_STAMP "Perl/%" SVf
+#endif
+    
     ap_add_version_component(pconf, MP_VERSION_STRING);
     ap_add_version_component(pconf,
-                             Perl_form(aTHX_ "Perl/v%vd", PL_patchlevel));
+                             Perl_form(aTHX_ MP_PERL_VERSION_STAMP,
+                                       PL_patchlevel));
+
     modperl_mgv_hash_handlers(pconf, s);
     modperl_modglobal_hash_keys(aTHX);
     modperl_env_hash_keys(aTHX);
