@@ -1,8 +1,9 @@
-/* Copyright 2000-2005 The Apache Software Foundation
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+/* Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -145,4 +146,7 @@ void modperl_package_unload(pTHX_ const char *package);
 void modperl_restart_count_inc(server_rec *base_server);
 int  modperl_restart_count(void);
 
+SV *modperl_pnotes(pTHX_ HV **pnotes, SV *key, SV *val,
+                   request_rec *r, conn_rec *c);
+
 #endif /* MODPERL_UTIL_H */
