@@ -9,15 +9,23 @@ use warnings FATAL => 'all';
 
 use Apache::TestUtil;
 use Apache::Test;
+
+use ModPerl::Util ();
+use Apache2::compat ();
+use Apache::Constants qw(DIR_MAGIC_TYPE OPT_EXECCGI :common :response);
+
 use File::Spec::Functions qw(catfile canonpath);
 
-use Apache::compat ();
-use Apache::Constants qw(DIR_MAGIC_TYPE :common :response);
+sub fixup {
+    my $r = shift;
+    Apache->httpd_conf('Options +ExecCGI');
+    OK;
+}
 
 sub handler {
     my $r = shift;
 
-    plan $r, tests => 17;
+    plan $r, tests => 24;
 
     $r->send_http_header('text/plain');
 
@@ -26,16 +34,15 @@ sub handler {
     ok t_cmp(ref($fh), 'GLOB', "Apache->gensym");
 
     ok t_cmp(Apache->module('mod_perl.c'), 1,
-             "Apache::module('mod_perl.c')");
+             "Apache2::module('mod_perl.c')");
     ok t_cmp(Apache->module('mod_ne_exists.c'), 0,
-             "Apache::module('mod_ne_exists.c')");
-
+             "Apache2::module('mod_ne_exists.c')");
 
     ok t_cmp(Apache->define('MODPERL2'),
-             Apache::ServerUtil::exists_config_define('MODPERL2'),
+             Apache2::ServerUtil::exists_config_define('MODPERL2'),
              'Apache->define');
 
-    ok t_cmp(Apache::current_callback(),
+    ok t_cmp($r->current_callback,
              'PerlResponseHandler',
              'inside PerlResponseHandler');
 
@@ -43,6 +50,22 @@ sub handler {
     Apache::log_error("Apache::log_error test ok");
     ok 1;
 
+    t_server_log_warn_is_expected();
+    Apache->warn('Apache->warn ok');
+    ok 1;
+
+    t_server_log_warn_is_expected();
+    Apache::warn('Apache::warn ok');
+    ok 1;
+
+    t_server_log_warn_is_expected();
+    Apache::Server->warn('Apache::Server->warn ok');
+    ok 1;
+
+    t_server_log_warn_is_expected();
+    Apache::Server::warn('Apache::Server::warn ok');
+    ok 1;
+
     # explicitly imported
     ok t_cmp(DIR_MAGIC_TYPE, "httpd/unix-directory",
              'DIR_MAGIC_TYPE');
@@ -58,11 +81,8 @@ sub handler {
     ok t_cmp(OK, "0",
              'OK');
 
-    my $admin = $r->server->server_admin;
-    Apache->httpd_conf('ServerAdmin foo@bar.com');
-    ok t_cmp($r->server->server_admin, 'foo@bar.com',
-             'Apache->httpd_conf');
-    Apache->httpd_conf("ServerAdmin $admin");
+    my $exec_cgi = $r->allow_options & Apache2::Const::OPT_EXECCGI;
+    ok t_cmp($exec_cgi, Apache2::Const::OPT_EXECCGI, 'Apache->httpd_conf');
 
     # (Apache||$r)->server_root_relative
     {
@@ -81,18 +101,25 @@ sub handler {
 
         ok t_filepath_cmp(canonpath(Apache->server_root_relative('conf')),
                           catfile($server_root, 'conf'),
-                          "Apache->server_root_relative('conf')");
+                          "Apache2::ServerUtil->server_root_relative('conf')");
 
         ok t_filepath_cmp(canonpath(Apache->server_root_relative),
                           canonpath($server_root),
-                          'Apache->server_root_relative()');
+                          'Apache2::ServerUtil->server_root_relative()');
 
-        my $path = catfile(Apache::ServerUtil::server_root, 'logs');
+        my $path = catfile(Apache2::ServerUtil::server_root, 'logs');
         ok t_filepath_cmp(canonpath(Apache->server_root_relative($path)),
                           canonpath($path),
                           "Apache->server_root_relative('$path')");
     }
 
+    ok t_cmp(Apache->unescape_url_info("/foo+bar%20baz"),
+             '/foo bar baz',
+             'Apache->unescape_url_info');
+
+    ok t_cmp $Apache::Server::Starting,   0, '$Apache::Server::Starting';
+    ok t_cmp $Apache::Server::ReStarting, 1, '$Apache::Server::ReStarting';
+
     OK;
 }
 
@@ -100,4 +127,7 @@ sub handler {
 
 __END__
 # so we can test whether send_httpd_header() works fine
-PerlOptions +ParseHeaders
+PerlOptions +ParseHeaders +GlobalRequest
+AllowOverride Options
+PerlModule TestCompat::apache
+PerlFixupHandler TestCompat::apache::fixup
